@@ -2,11 +2,13 @@
 #' @param x a factor
 #' @export
 is_prefixed <- function(x) {
-  if (!is.factor(x)) {
-    stop("is_prefixed should be used only with a factor.")
-  }
+  if (!is.factor(x))
+    cli::cli_abort(paste(
+      "{.fn is_prefixed} should be used only with a factor",
+      "({.arg x} is {class(x)})."
+    ))
   l <- .get_prefixes.factor(x)
-  all(!is.na(l$code)) && all(!is.na(l$code)) && !any(duplicated(l$code))
+  !anyNA(l$code) && !anyNA(l$code) && !any(duplicated(l$code))
 }
 
 
