@@ -2,22 +2,35 @@
 #'
 #' @param x a vector or a data.frame
 #' @param value a character string or `NULL` to remove the label
-#'  For data frames, it could also be a named list or a character vector
-#'  of same length as the number of columns in `x`.
+#'  For data frames, with `var_label()`, it could also be a named list or a
+#'  character vector of same length as the number of columns in `x`.
 #' @param unlist for data frames, return a named vector instead of a list
 #' @param null_action for data frames, by default `NULL` will be returned for
 #' columns with no variable label. Use `"fill"` to populate with the column name
-#' instead, or `"skip"` to remove such values from the returned list.
+#' instead, `"skip"` to remove such values from the returned list, `"na"` to
+#' populate with `NA` or `"empty"` to populate with an empty string (`""`).
+#' @param recurse if `TRUE`, will apply `var_label()` on packed columns
+#' (see [tidyr::pack()]) to return the variable labels of each sub-column;
+#' otherwise, the label of the group of columns will be returned.
 #' @details
-#'   For data frames, if `value` is a named list, only elements whose name will
-#'   match a column of the data frame will be taken into account. If `value`
-#'   is a character vector, labels should in the same order as the columns of
-#'   the data.frame.
+#'   `get_variable_labels()` is identical to `var_label()`.
+#'
+#'   For data frames, if you are using `var_label()<-` and if `value` is a
+#'   named list, only elements whose name will match a column of the data frame
+#'   will be taken into account. If `value` is a character vector, labels should
+#'   be in the same order as the columns of the data.frame.
+#'
+#'   If you are using `label_attribute()<-` or `set_label_attribute()` on a data
+#'   frame, the label attribute will be attached to the data frame itself, not
+#'   to a column of the data frame.
+#'
+#'   If you are using packed columns (see [tidyr::pack()]), please read the
+#'   dedicated vignette.
 #' @examples
 #' var_label(iris$Sepal.Length)
-#' var_label(iris$Sepal.Length) <- 'Length of the sepal'
+#' var_label(iris$Sepal.Length) <- "Length of the sepal"
 #' \dontrun{
-#'  View(iris)
+#' View(iris)
 #' }
 #' # To remove a variable label
 #' var_label(iris$Sepal.Length) <- NULL
@@ -25,7 +38,7 @@
 #' var_label(iris) <- c(
 #'   "sepal length", "sepal width", "petal length",
 #'   "petal width", "species"
-#')
+#' )
 #' var_label(iris)
 #' var_label(iris) <- list(
 #'   Petal.Width = "width of the petal",
@@ -38,13 +51,17 @@
 #' var_label(iris, null_action = "skip")
 #' var_label(iris, unlist = TRUE)
 #'
-#
+#' #
 #' @export
 var_label <- function(x, ...) {
   rlang::check_dots_used()
   UseMethod("var_label")
 }
 
+var_label_no_check <- function(x, ...) {
+  UseMethod("var_label")
+}
+
 #' @export
 var_label.default <- function(x, ...) {
   attr(x, "label", exact = TRUE)
@@ -52,9 +69,23 @@ var_label.default <- function(x, ...) {
 
 #' @rdname var_label
 #' @export
-var_label.data.frame <- function(x, unlist = FALSE,
-                                 null_action = c("keep", "fill", "skip"), ...) {
-  r <- lapply(x, var_label)
+var_label.data.frame <- function(x,
+                                 unlist = FALSE,
+                                 null_action =
+                                   c("keep", "fill", "skip", "na", "empty"),
+                                 recurse = FALSE,
+                                 ...) {
+  if (recurse) {
+    r <- lapply(
+      x,
+      var_label_no_check,
+      unlist = unlist,
+      null_action = null_action,
+      recurse = TRUE
+    )
+  } else {
+    r <- lapply(x, label_attribute)
+  }
 
   null_action <- match.arg(null_action)
 
@@ -69,6 +100,24 @@ var_label.data.frame <- function(x, unlist = FALSE,
     )
   }
 
+  if (null_action == "empty") {
+    r <- lapply(
+      r,
+      function(x) {
+        if (is.null(x)) "" else x
+      }
+    )
+  }
+
+  if (null_action == "na") {
+    r <- lapply(
+      r,
+      function(x) {
+        if (is.null(x)) NA_character_ else x
+      }
+    )
+  }
+
   if (null_action == "skip") {
     r <- r[!sapply(r, is.null)]
   }
@@ -94,24 +143,21 @@ var_label.data.frame <- function(x, unlist = FALSE,
 
 #' @export
 `var_label<-.default` <- function(x, value) {
-  if ((!is.character(value) && !is.null(value)) || length(value) >
-    1)
-    stop("`value` should be a single character string or NULL",
-      call. = FALSE, domain = "R-labelled")
-  attr(x, "label") <- value
+  label_attribute(x) <- value
   x
 }
 
 #' @export
 `var_label<-.data.frame` <- function(x, value) {
-  if ((!is.character(value) && !is.null(value)) && !is.list(value) ||
-    (is.character(value) && length(value) > 1 && length(value) != ncol(x)))
-    stop(
-      paste0(
-        "`value` should be a named list, NULL, a single character string or a ",
-        "character vector of same length than the number of columns in `x`"
-      ),
-      call. = FALSE, domain = "R-labelled")
+  if (
+    (!is.character(value) && !is.null(value)) && !is.list(value) ||
+      (is.character(value) && length(value) > 1 && length(value) != ncol(x))
+  )
+    cli::cli_abort(paste(
+      "{.arg value} should be a named list, NULL, a single character string",
+      "or a character vector of same length than the number of columns",
+      "in {.arg x}."
+    ))
   if (is.character(value) && length(value) == 1) {
     value <- as.list(rep(value, ncol(x)))
     names(value) <- names(x)
@@ -129,18 +175,22 @@ var_label.data.frame <- function(x, unlist = FALSE,
   }
 
   if (!all(names(value) %in% names(x))) {
-    missing_names <- stringr::str_c(
-      setdiff(names(value), names(x)),
-      collapse = ", "
-    )
-    stop("some variables not found in x:", missing_names)
+    missing_names <- setdiff(names(value), names(x))
+
+    cli::cli_abort(c(
+      "Can't find variables {.var {missing_names}} in {.arg x}."
+    ))
   }
 
   value <- value[names(value) %in% names(x)]
-  for (var in names(value)) var_label(x[[var]]) <- value[[var]]
+  for (var in names(value)) label_attribute(x[[var]]) <- value[[var]]
   x
 }
 
+#' @rdname var_label
+#' @export
+get_variable_labels <- var_label
+
 
 #' @rdname var_label
 #' @param .data a data frame or a vector
@@ -166,15 +216,17 @@ var_label.data.frame <- function(x, unlist = FALSE,
 #'
 #'   # Set labels from dictionary, e.g. as read from external file
 #'   # One description is missing, one has no match
-#'   description = tibble(
+#'   description <- tibble(
 #'     name = c(
 #'       "Sepal.Length", "Sepal.Width", "Petal.Length", "Petal.Width",
-#'       "Something"),
+#'       "Something"
+#'     ),
 #'     label = c(
-#'       "Sepal length", "Sepal width",  "Petal length", "Petal width",
-#'       "something")
+#'       "Sepal length", "Sepal width", "Petal length", "Petal width",
+#'       "something"
+#'     )
 #'   )
-#'   var_labels <- setNames(as.list(description$label), description$name)
+#'   var_labels <- stats::setNames(as.list(description$label), description$name)
 #'   iris_labelled <- iris %>%
 #'     set_variable_labels(.labels = var_labels, .strict = FALSE)
 #'   var_label(iris_labelled)
@@ -194,38 +246,61 @@ var_label.data.frame <- function(x, unlist = FALSE,
 #' }
 #' @export
 set_variable_labels <- function(.data, ..., .labels = NA, .strict = TRUE) {
-  if (!is.data.frame(.data) && !is.atomic(.data))
-    stop(".data should be a data.frame or a vector")
-
-  # vector case
-  if (is.atomic(.data)) {
+  # not a data.frame
+  if (!is.data.frame(.data)) {
     if (!identical(.labels, NA)) {
-      var_label(.data) <- .labels
+      label_attribute(.data) <- .labels
     } else {
-      var_label(.data) <- unname(unlist(rlang::dots_list(...)))
+      label_attribute(.data) <- unname(unlist(rlang::dots_list(...)))
     }
     return(.data)
   }
 
   # data.frame case
   if (!identical(.labels, NA)) {
-    if (!.strict)
+    if (!.strict) {
       .labels <- .labels[intersect(names(.labels), names(.data))]
+    }
     var_label(.data) <- .labels
   }
   values <- rlang::dots_list(...)
   if (length(values) > 0) {
     if (.strict && !all(names(values) %in% names(.data))) {
-      missing_names <- stringr::str_c(
-        setdiff(names(values), names(.data)),
-        collapse = ", "
-      )
-      stop("some variables not found in .data: ", missing_names)
+      missing_names <- setdiff(names(values), names(.data))
+
+      cli::cli_abort(c(
+        "Can't find variables {.var {missing_names}}  in {.arg .data}."
+      ))
     }
 
-    for (v in intersect(names(values), names(.data)))
-      var_label(.data[[v]]) <- values[[v]]
+    for (v in intersect(names(values), names(.data))) {
+      label_attribute(.data[[v]]) <- values[[v]]
+    }
   }
 
   .data
 }
+
+#' @rdname var_label
+#' @export
+label_attribute <- function(x) {
+  attr(x, "label", exact = TRUE)
+}
+
+#' @rdname var_label
+#' @export
+get_label_attribute <- function(x) {
+  label_attribute(x)
+}
+
+#' @rdname var_label
+#' @export
+set_label_attribute <- function(x, value) {
+  check_string(value, allow_null = TRUE, allow_na = TRUE)
+  attr(x, "label") <- value
+  x
+}
+
+#' @rdname var_label
+#' @export
+`label_attribute<-` <- set_label_attribute
