@@ -1,5 +1,3 @@
-context("Labelled")
-
 # var_label --------------------------------------------------------------
 
 test_that("var_label works properly", {
@@ -10,6 +8,7 @@ test_that("var_label works properly", {
   var_label(x) <- NULL
   expect_null(attr(x, "label"))
   expect_null(var_label(x))
+  expect_no_error(var_label(x) <- NA_character_)
 
   x <- 1:3
   x <- set_variable_labels(x, "value")
@@ -18,6 +17,22 @@ test_that("var_label works properly", {
   expect_equal(attr(x, "label"), "other value")
   x <- set_variable_labels(x, NULL)
   expect_null(attr(x, "label"))
+
+  labs <- list(speed = NULL, dist = NULL)
+  expect_no_error(
+    set_variable_labels(cars, .labels = labs)
+  )
+  expect_no_error(
+    var_label(cars) <- labs
+  )
+
+  labels <- list(dist = "BAR", speed = "FOO")
+  data_labelled <- set_variable_labels(cars, .labels = labels)
+
+  expect_identical(
+    get_variable_labels(data_labelled),
+    list(speed = "FOO", dist = "BAR")
+  )
 })
 
 test_that("var_label works on data.frame", {
@@ -51,15 +66,13 @@ test_that("var_label produce appropriate errors", {
       set_variable_labels(x = "ghj", z = "ggg")
   )
   # no error if .strict = FALSE
-  expect_error(
+  expect_no_error(
     df %>%
-      set_variable_labels(.labels = list(x = "xx", z = "zz"), .strict = FALSE),
-    NA
+      set_variable_labels(.labels = list(x = "xx", z = "zz"), .strict = FALSE)
   )
-  expect_error(
+  expect_no_error(
     df %>%
-      set_variable_labels(x = "ghj", z = "ggg", .strict = FALSE),
-    NA
+      set_variable_labels(x = "ghj", z = "ggg", .strict = FALSE)
   )
 })
 
@@ -86,8 +99,8 @@ test_that("x must be numeric or character", {
 
 test_that("x and labels must be compatible", {
   expect_error(labelled(1, "a"))
-  expect_error(labelled(1, c(female = 2L, male = 1L)), NA)
-  expect_error(labelled(1L, c(female = 2, male = 1)), NA)
+  expect_no_error(labelled(1, c(female = 2L, male = 1L)))
+  expect_no_error(labelled(1L, c(female = 2, male = 1)))
 })
 
 test_that("labels must have names", {
@@ -150,6 +163,16 @@ test_that("value labels can be removed if missing values are defined", {
   expect_null(val_labels(x))
 })
 
+test_that("val_labels() null action", {
+  x <- labelled(1:10, c(Good = 1, Bad = 8))
+
+  val_labels(x, null_action = "labelled") <- NULL
+  expect_true(inherits(x, "haven_labelled"))
+
+  val_labels(x) <- NULL
+  expect_false(inherits(x, "haven_labelled"))
+})
+
 test_that("value labels to NULL remove class if na_Values et na_range are NULL", { # nolint
   x <- labelled_spss(1:10, c(Good = 1, Bad = 8))
   val_labels(x) <- NULL
@@ -234,7 +257,7 @@ test_that(" 'val_labels <-'  works for dataframe", {
     num = c(two = 2)
   )
   vldf <- df
-  expect_error(val_labels(vldf) <- valeurs, NA)
+  expect_no_error(val_labels(vldf) <- valeurs)
 
   expect_null(val_labels(vldf)$fac)
   expect_equal(df$fac, vldf$fac)
@@ -324,6 +347,10 @@ test_that(" 'val_label<-' works properly", {
   expect_error(val_label(df, 2:3) <- "a")
 
   sub_df <- df[, -match("ch", names(df))]
+
+  v <- as.Date("2023-01-01")
+  l <- as.Date(c("The first day of 2023" = "2023-01-01"))
+  expect_error(val_labels(v) <- l)
 })
 
 test_that(" 'val_label<-.data.frame' works properly", {
@@ -377,7 +404,7 @@ test_that("remove_labels strips labelled attributes", {
   expect_equal(remove_labels(var), exp)
 })
 
-test_that("remove_labels returns variables not of class('labelled') unmodified", { #nolint
+test_that("remove_labels returns variables not of class('labelled') unmodified", { # nolint
   var <- c(1L, 98L, 99L)
   expect_equal(remove_labels(var), var)
 })
@@ -396,7 +423,7 @@ test_that("remove_labels works with labelled_spss", {
   xhs <- haven::labelled_spss(
     c(1, 2, 3, NA, 99),
     c(t1 = 1, t2 = 2, Missing = 99),
-    na_value = 99,
+    na_values = 99,
     na_range = c(99, Inf),
     label = "A test variable"
   )
@@ -472,14 +499,15 @@ test_that("sort_val_labels works properly", {
   x <- c(2, tagged_na("z"), 1, tagged_na("a"))
   val_labels(x) <-
     c(no = 2, refused = tagged_na("z"), yes = 1, dk = tagged_na("a"))
-  expect_equivalent(
+  expect_equal(
     sort_val_labels(x, according_to = "v") %>%
       val_labels() %>%
       format_tagged_na() %>%
       trimws(),
-    c("1", "2", "NA(a)", "NA(z)")
+    c("1", "2", "NA(a)", "NA(z)"),
+    ignore_attr = "names"
   )
-  expect_equivalent(
+  expect_equal(
     sort_val_labels(x, according_to = "l") %>% val_labels() %>% names(),
     c("dk", "no", "refused", "yes")
   )
@@ -498,7 +526,7 @@ test_that("remove_user_na works properly", {
   xhs <- haven::labelled_spss(
     c(1, 2, NA, 98, 99),
     c(t1 = 1, t2 = 2, Missing = 99),
-    na_value = 99,
+    na_values = 99,
     na_range = c(99, Inf),
     label = "A test variable"
   )
@@ -572,7 +600,7 @@ test_that("to_factor boolean parameters", {
   x1 <- haven::labelled_spss(
     c(1, 2, 3, 5, 4, NA, 99),
     c(t1 = 1, t2 = 2, t5 = 5, Missing = 99),
-    na_value = 99
+    na_values = 99
   )
 
   tfx <- to_factor(x1, user_na_to_na = TRUE)
@@ -595,7 +623,7 @@ test_that("to_factor parameters : sort_levels + levels", {
   x1 <- haven::labelled_spss(
     c(1, 2, 3, 5, 4, NA, 99),
     c(t1 = 1, t2 = 2, t5 = 5, Missing = 99),
-    na_value = 99
+    na_values = 99
   )
 
   tfx <- to_factor(x1, sort_levels = "auto")
@@ -634,13 +662,15 @@ test_that("to_factor() and tagged NAs", {
   expect_equal(
     to_factor(x),
     structure(c(1L, 2L, NA, 1L, NA, 2L, NA, NA),
-              .Label = c("yes", "no"), class = "factor")
+      .Label = c("yes", "no"), class = "factor"
+    )
   )
   expect_equal(
     to_factor(x, explicit_tagged_na = TRUE),
     structure(c(1L, 2L, 4L, 1L, 5L, 2L, 4L, 3L),
-              .Label = c("yes", "no", "toto", "missing", "NA(z)"),
-              class = "factor")
+      .Label = c("yes", "no", "toto", "missing", "NA(z)"),
+      class = "factor"
+    )
   )
 })
 
@@ -700,6 +730,10 @@ test_that("set_value_labels replaces all value labels", {
   expect_equal(val_labels(df$s2), c(Yes = 1, No = 2))
   df <- set_value_labels(df, s2 = c(Yes = 1, Unknown = 9))
   expect_equal(val_labels(df$s2), c(Yes = 1, Unknown = 9))
+  df <- set_value_labels(df, s1 = NULL)
+  df <- set_value_labels(df, s2 = NULL, .null_action = "lab")
+  expect_false(inherits(df$s1, "haven_labelled"))
+  expect_true(inherits(df$s2, "haven_labelled"))
 
   v <- set_value_labels(1:10, c(low = 1, high = 10))
   expect_equal(val_labels(v), c(low = 1, high = 10))
@@ -734,16 +768,15 @@ test_that("set_value_labels errors", {
       )
   )
   # no error if .strict = FALSE
-  expect_error(
+  expect_no_error(
     df %>%
       set_value_labels(
         s1 = c(Male = "M", Female = "F"),
         s3 = c(Yes = 1, No = 2),
         .strict = FALSE
-      ),
-    NA
+      )
   )
-  expect_error(
+  expect_no_error(
     df %>%
       set_value_labels(
         .labels = list(
@@ -751,8 +784,7 @@ test_that("set_value_labels errors", {
           s3 = c(Yes = 1, No = 2)
         ),
         .strict = FALSE
-      ),
-    NA
+      )
   )
 })
 
@@ -779,16 +811,15 @@ test_that("add_value_labels errors", {
       )
   )
   # no error if .strict = FALSE
-  expect_error(
+  expect_no_error(
     df %>%
       add_value_labels(
         s1 = c(Male = "M", Female = "F"),
         s3 = c(Yes = 1, No = 2),
         .strict = FALSE
-      ),
-    NA
+      )
   )
-  expect_error(
+  expect_no_error(
     df %>%
       add_value_labels(
         .labels = list(
@@ -796,8 +827,7 @@ test_that("add_value_labels errors", {
           s3 = c(Yes = 1, No = 2)
         ),
         .strict = FALSE
-      ),
-    NA
+      )
   )
 
   expect_error(add_value_labels(df, s1 = c("F", Male = "M")))
@@ -843,7 +873,7 @@ test_that("set_variable_labels updates variable labels", {
 
 # missing values --------------------------------------------------------------
 
-test_that("it is possible to define missing values if no value labels were defined", {# nolint
+test_that("it is possible to define missing values if no value labels were defined", { # nolint
   x <- c(1, 2, 2, 9)
   na_values(x) <- 9
   expect_equal(na_values(x), 9)
@@ -876,6 +906,17 @@ test_that("dplyr::recode could be applied to character labelled vector", {
   expect_equal(x, labelled(c("a", "b", "b"), c(yes = "a", no = "b")))
 })
 
+test_that("dplyr::recode could handle NA with .combine_value_labels", {
+  x <- labelled(c(NA, 1:3), c(yes = 1, maybe = 2, no = 3))
+  y <- x %>% dplyr::recode(`2` = 0L, .combine_value_labels = TRUE)
+  expect_true(all(c(0, 1, 3) %in% val_labels(y)))
+
+  y <- x %>% dplyr::recode(`2` = 0L, `3` = 0L, .combine_value_labels = TRUE)
+  expect_true(all(c(0, 1) %in% val_labels(y)))
+  expect_equal(val_label(y, 0), "maybe / no")
+})
+
+
 # update_labelled ----------------------------------------
 
 test_that("update_labelled update previous haven's labelled objects but not Hmisc's labelled objects", { # nolint
@@ -895,7 +936,7 @@ test_that("update_labelled update previous haven's labelled objects but not Hmis
   expect_s3_class(update_labelled(df)$vHmisc, "labelled")
 })
 
-test_that("update_labelled update to haven_labelled_spss if there are na values", { #nolint
+test_that("update_labelled update to haven_labelled_spss if there are na values", { # nolint
   v1 <- structure(1:4,
     label = "label", labels = c(No = 1, Yes = 2),
     na_values = c(8, 9), class = c("labelled_spss", "labelled")
@@ -1015,7 +1056,8 @@ test_that("dplyr::recode works properly with labelled vectors", {
   x <- labelled(1:4, c(a = 1, b = 2, c = 3, d = 4))
 
   r <- dplyr::recode(
-    x, `1` = 1L, `2` = 1L, `3` = 2L, `4` = 2L,
+    x,
+    `1` = 1L, `2` = 1L, `3` = 2L, `4` = 2L,
     .combine_value_labels = TRUE
   )
   expect_equal(val_labels(r), c("a / b" = 1L, "c / d" = 2L))
@@ -1024,7 +1066,8 @@ test_that("dplyr::recode works properly with labelled vectors", {
   expect_equal(val_labels(r), c("a / b" = 1L, "c / d" = 3L))
 
   r <- dplyr::recode(
-    x, `2` = 1L, `4` = 3L,
+    x,
+    `2` = 1L, `4` = 3L,
     .combine_value_labels = TRUE,
     .sep = " or "
   )
@@ -1133,12 +1176,13 @@ test_that("names_prefixed_by_values works properly", {
     c1 = c("[M] Male", "[F] Female"),
     c2 = c("[1] Yes", "[2] No")
   )
-  expect_equivalent(
+  expect_equal(
     names_prefixed_by_values(val_labels(df)),
-    res_names_prefixed
+    res_names_prefixed,
+    ignore_attr = "names"
   )
 
-  expect_true(is.null(names_prefixed_by_values(NULL)))
+  expect_null(names_prefixed_by_values(NULL))
 })
 
 test_that("null_action in var_label() works as expected", {
@@ -1167,6 +1211,26 @@ test_that("null_action in var_label() works as expected", {
       Species = "Species"
     )
   )
+  expect_equal(
+    var_label(df, null_action = "na"),
+    list(
+      Sepal.Length = NA_character_,
+      Sepal.Width = NA_character_,
+      Petal.Length = "length of petal",
+      Petal.Width = "width of petal",
+      Species = NA_character_
+    )
+  )
+  expect_equal(
+    var_label(df, null_action = "empty"),
+    list(
+      Sepal.Length = "",
+      Sepal.Width = "",
+      Petal.Length = "length of petal",
+      Petal.Width = "width of petal",
+      Species = ""
+    )
+  )
   expect_equal(
     var_label(df, null_action = "skip"),
     list(
@@ -1176,3 +1240,44 @@ test_that("null_action in var_label() works as expected", {
   )
   expect_error(var_label(df$Species, null_action = "skip"))
 })
+
+test_that("var_label works with packed columns", {
+  d <- iris %>%
+    tidyr::as_tibble() %>%
+    tidyr::pack(
+      Sepal = starts_with("Sepal"),
+      Petal = starts_with("Petal"),
+      .names_sep = "."
+    )
+
+  d <- d %>% set_variable_labels(Sepal = "Label of the Sepal df-column")
+  expect_equal(
+    label_attribute(d$Sepal),
+    "Label of the Sepal df-column"
+  )
+
+  d$Petal <- d$Petal %>%
+    set_variable_labels(
+      Length = "Petal length",
+      Width = "Petal width"
+    )
+  expect_equal(
+    label_attribute(d$Petal$Length),
+    "Petal length"
+  )
+
+  expect_equal(
+    length(var_label(d)),
+    3L
+  )
+
+  expect_equal(
+    length(var_label(d, recurse = TRUE)),
+    3L
+  )
+
+  expect_equal(
+    length(var_label(d, recurse = TRUE, unlist = TRUE)),
+    5L
+  )
+})
